#version 430 core
/*
* Copyright 2018 Vienna University of Technology.
* Institute of Computer Graphics and Algorithms.
* This file is part of the ECG Lab Framework and must not be redistributed.
*/

#define MAX_Explosions 10

uniform float numExpl;
//not int because our setUniform has issues with int

layout(location = 0) in vec3 position;
layout(location = 1) in vec3 normal;
layout(location = 2) in vec2 texCoord;

uniform mat4 modelMatrix;
uniform float angle;
layout(location = 3) uniform mat4 viewMatrix;
layout(location = 4) uniform mat4 projMatrix;

out vec3 position_world;
out vec3 newNormal;
out vec2 TexCoord;

struct Explosion {
	vec3 origin;
	float timeSinceExplosion;
};

uniform Explosion explosion[MAX_Explosions];
float exSpeed = 400.0f;
float exHeight = 2.0f;
float exWidth = 50.0f;
float exMaxTime = 1.0f;

bool inRange(float v, float rs, float re);
float easeInOutQuat(float t);

void main() {
	vec4 pos = modelMatrix * vec4(position, 1);
	
	for(int i = 0; i < numExpl; i++) {
		float tse = explosion[i].timeSinceExplosion;
		float exRadius = tse * exSpeed;
		float distToOrigin = length(explosion[i].origin - vec3(pos));

		if(inRange(distToOrigin, exRadius - exWidth / 2 ,exRadius + exWidth/2) && tse < exMaxTime) {
			pos.y += easeInOutQuat(1 - abs(distToOrigin - exRadius) / (exWidth / 2))
						* exHeight
						* easeInOutQuat(1 - (tse) / exMaxTime);
		}
		
	}
	
	
	gl_Position = projMatrix * viewMatrix * pos;
	
	

	vec4 transformedNormal = transpose(inverse(modelMatrix)) * vec4(normal, 1);

	newNormal = normalize(transformedNormal.xyz);
	position_world = (modelMatrix * vec4(position, 1)).xyz;
	TexCoord = texCoord;
}

bool inRange(float v, float rs, float re) {
	return rs < v && v < re;
}

//from https://gist.github.com/gre/1650294
float easeInOutQuat(float t) {
	return t < 0.5f ? 4.0f*t*t*t : -1.0f+(4.0f-2.0f*t)*t;
}